void
on_open1_activate( GtkMenuItem *, gpointer);

void
on_save2_activate( GtkMenuItem *, gpointer);

void
on_save_as1_activate( GtkMenuItem *, gpointer);

void
on_mw_new_button_released( GtkButton *, gpointer);

void
on_mw_open_button_released( GtkButton *, gpointer);

void
on_mw_save_button_released( GtkButton *, gpointer);

void
on_fe_msgbox1_clicked( GnomeDialog *, gint, gpointer);

void
on_fe_msgbox2_clicked( GnomeDialog *, gint, gpointer);

void
on_file_open_dlg_ok_button_released( GtkButton *, gpointer);

void
on_file_open_dlg_cancel_button_released( GtkButton *, gpointer);

void
on_file_save_dlg_ok_button_released( GtkButton *, gpointer);

void
on_file_save_dlg_cancel_button_released( GtkButton *, gpointer);

void
on_screc_dlg_clicked( GnomeDialog *, gint, gpointer);
